/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include <iostream>
#include <stdio.h>
#include <string>


#include "qunitP.h"
#include "qunit_if.h"

// *************************************************************************

main( int argc, char **argv )
{
	static char local_buf[100];

	double data = 0.5;
	double conversion;
	double Mu0 = 1.25664e-06;
	double k = 1.38054e-23;
	
	std::string SI1 = "1.0e-9>T";
	std::string SI2 = "1.0>K";
	std::string SI3 = "1.0>(number) m^-3";
	std::string SIMu0 = "1>H m^-1";
	std::string SIk = "1>J K^-1";
	std::string SInPascal ;
	std::string magneticP;
	std::string ionP;
	
	// QunitMakeSIconvStr()
	//----------------------
	
	SInPascal = QunitMakeSIconvStr(1e-9, "Pa m m^-1"); // = "1e-9>Pa"
	std::cout <<  "\n Start QUNIT Test...\n\nOn creation of SIconv, '1e-9, Pa m m^-1' becomes '" << SInPascal.c_str() << "'" << std::endl;

	// QunitHasUnits
	// -------------
	std::cout <<  "\nIs this syntax good '" << SInPascal << "' is OK? '" << QunitHasUnits(SInPascal.c_str()) << "'" << std::endl;
	std::cout <<  "Note QunitMakeSIconvSrt() cleans up the SI units part \n\n" << std::endl;

	
	
	// QunitGetBaseSI()
	//------------------
	
	std::cout << "B field,          '" << SI1.c_str()  << "'          becomes in BASE SI '" << QunitGetBaseSI(SI1.c_str()) << "'" << std::endl;
	std::cout << "Temperature,      '" << SI2.c_str() << "'             becomes in BASE SI '" << QunitGetBaseSI(SI2.c_str()) << "'" << std::endl;
	std::cout << "Number Density,   '" << SI3.c_str() << "' becomes in BASE SI '" << QunitGetBaseSI(SI3.c_str()) << "'" << std::endl;
	std::cout << "Mu0,              '" << SIMu0.c_str() << "'          becomes in BASE SI '" << QunitGetBaseSI(SIMu0.c_str()) << "'" << std::endl;
	std::cout << "Boltzman's const, '" << SIk.c_str() << "'          becomes in BASE SI '" << QunitGetBaseSI(SIk.c_str()) << "'" << std::endl;
	std::cout << "Pascal,           '" << SInPascal.c_str() << "'        becomes in BASE SI '" << QunitGetBaseSI(SInPascal.c_str()) << "'" << std::endl;
	
	// QunitConvFactor()
	//------------------
	
	// SI for field squared
	double newConv = QunitConvFactor(SI1.c_str());
	newConv *= newConv;

	sprintf(local_buf, "%g >", newConv);
    magneticP = std::string(local_buf);
	
	// QunitUnitsStr()
	//--------------
	magneticP += QunitUnitsStr(SI1.c_str()) + " ";
	magneticP += QunitUnitsStr(SI1.c_str());
	
	// QunitGetTidySI()
	//-----------------
	
	std::cout << "\nUsing QunitGetTidySI()...\n'" << magneticP.c_str() << "' becomes Tidy SI '" << QunitGetTidySI(magneticP.c_str()) << "'" << std::endl;
	
	// QunitGetProductSI()
	//------------------
	
	// This is easier with QunitGetProductSI()
	
	magneticP = QunitGetProductSI(SI1.c_str(), SI1.c_str()) ;
	std::cout << "\n More easily\n'" << SI1.c_str() << "' squared becomes Tidy SI '" << magneticP.c_str() << "'" << std::endl;

	// QunitGetPowerSI()
	// --------------------
	
	// divide by Mu to get mag pressure
	magneticP = QunitGetProductSI(magneticP.c_str(), QunitGetPowerSI(SIMu0.c_str(), -1).c_str()) ;
	std::cout << "\n Magnetic Pressure SIconv is '" << magneticP.c_str() << "'" << std::endl;

	// Ion pressure is nkT
	
	ionP = QunitGetProductSI(SI3.c_str(), SIk.c_str()) ; 
	ionP = QunitGetProductSI(ionP.c_str(), SI2.c_str()) ; 

	std::cout << "\n Ion Pressure SIconv is '" << ionP.c_str() << "'" << std::endl;

	// QunitAreUnitsSame() and QunitSameBaseSI()
	
	std::cout << "\n pressures not in same units... '" << QunitAreUnitsSame(ionP.c_str(), magneticP.c_str()) << "'" << std::endl;
	std::cout << "\n BUT pressures ARE in same BASE units... '" << QunitSameBaseSI(ionP.c_str(), magneticP.c_str()) << "'" << std::endl;

	std::cout << "\n so '" << ionP.c_str() << "' can be converted to '" << magneticP.c_str() << "'" << std::endl;
	std::cout << " since '" << QunitGetBaseSI(ionP.c_str()).c_str() << "' can be converted to '" << QunitGetBaseSI(magneticP.c_str()).c_str() << "'" << std::endl;
	std::cout << " by multiplying the ion pressure by '" << QunitConversion(ionP.c_str(), magneticP.c_str()) << "'" << std::endl;

	std::cout << "\n ...or ion P can be converted to nPa by multiplying by '" << QunitConversion(ionP.c_str(), "1e-9>Pa") << "'" << std::endl;
	std::cout << "   and mag P can be converted to nPa by multiplying by '" << QunitConversion(magneticP.c_str(), "1e-9>Pa") << "'" << std::endl;

	std::cout << "\nTest Complete\n\n";
	
	return 0;

}

